#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import libevh.evhcmd as evhcmd
import evh_config as config
from fal_info import (Results, ReqTypes)


EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"

SYSTEM_CATEGORY = "system"
RAID_CATEGORY = "raid"
VOLUME_CATEGORY = "volume"
INITIALIZE_ACTION = "initialize"
CREATE_ACTION = "create"
MKFSPREPARE_ACTION = "mkfsprepare"

PREVIOUS_ATTR = "previous"
ACTION_ATTR = "action"
DEV_KEY = "dev"
RAID_NAME_PATH = "raid/name"
VOLUME_NAME_PATH = "volume/name"


class EventHandler(evhcmd.EvhCmd):
    """Event Handler for the share of logext package."""

    def __handle_system_event(self, eventinfo, entry):
        """Handle system event."""

        if not eventinfo.is_set_to_true(PREVIOUS_ATTR):
            entry.set_result(Results.OK)
            return entry

        action = eventinfo.get_attr_value(ACTION_ATTR)

        if action == INITIALIZE_ACTION:
            entry.set_result(Results.EVENT_ERROR)
            return entry

        else:
            entry.set_result(Results.OK)

        return entry

    def __handle_raid_event(self, eventinfo, entry):
        """Handle raid event."""

        if not eventinfo.is_set_to_true(PREVIOUS_ATTR):
            entry.set_result(Results.OK)
            return entry

        action = eventinfo.get_attr_value(ACTION_ATTR)

        if action == CREATE_ACTION:
            if eventinfo.get_value(RAID_NAME_PATH) == config.INTERNAL_DEV:
                entry.set_result(Results.EVENT_ERROR)
                return entry

        else:
            entry.set_result(Results.OK)

        return entry

    def __handle_volume_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)

        if action == MKFSPREPARE_ACTION:
            if eventinfo.get_value(VOLUME_NAME_PATH) == config.INTERNAL_VOL:
                entry.set_result(Results.EVENT_ERROR)
                return entry

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == SYSTEM_CATEGORY:
                return self.__handle_system_event(eventinfo, entry)
            elif eventinfo.get_attr_value(CATEGORY_ATTR) == RAID_CATEGORY:
                return self.__handle_raid_event(eventinfo, entry)
            elif eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volume_event(eventinfo, entry)
        except:
            # Ignore all error.
            self.exception("logext force error")
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EventHandler(option=evhcmd.EvhCmdOption()).handle_request()
