#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
#from __future__ import absolute_import
from __future__ import division

import sys
sys.dont_write_bytecode = True

from optparse import OptionParser
import bdrec
from flocker import FileLocker
import uuid
import os
import codecs
import shutil
import time
import datetime
from compat import __init, __to_fspath
from sudo import __exec_cmd, __touch, __rmtree, __makedirs, __copytree
from keyvalue import __to_keyvalue, __to_keyvalue_str

LOG_DIR_DEFAULT = "/mnt/sataraid1/share/LAN DISK Log/Meimeikun"

DEV_ALT_NAME_DEFAULT = "DISC"
SRC_ALT_NAME_DEFAULT = "LAN DISK"

LOCK_FILE = "/var/lock/bdrec_meimeikun.lock"

FILE_REC_TYPE = "/var/run/meimeikun_bdrec.rec_type"

MOUNT_CMD = "/bin/mount"
UMOUNT_CMD = "/bin/umount"

ROOTFS = "/var/tmp/bdrec_meimeikun/rootfs"
ROOTFS_MNT_MEDIA = "/var/tmp/bdrec_meimeikun/rootfs_mnt_media"
ROOTFS_MNT_SHARE = "/var/tmp/bdrec_meimeikun/rootfs_mnt_share"
MEDIAINFO = ".mediainfo"
FISCALYEAR = ".fiscal_year"
MEDIAID = ".media_id"
CLOSEDISC = ".close_disc"
DATETIME = ".datetime"

ENABLE_FAKE_DISC = False

VOLID_FAKE_BLANK = "49cf6808af494135885444c66e49e4b4"

REC_TYPE_FAKE_BLANK = "fake_blank"
REC_TYPE_FORMAT = "format"
REC_TYPE_WRITE = "write"
REC_TYPE_WRITE_WITH_CLOSE = REC_TYPE_WRITE + " with close"

REC_RESULT_SUCCESS = 0
REC_RESULT_FAILED = 1
REC_RESULT_DEV_NOT_READY = 2
REC_RESULT_MEDIA_NOT_BLANK = 3
REC_RESULT_MEDIA_NOT_APPENDABLE = 4
REC_RESULT_MEDIA_INVALID_FORMAT = 5
REC_RESULT_MEDIA_ALREADY_CLOSED = 6
REC_RESULT_MEDIA_FISCAL_YEAR_NOT_MATCH = 7
REC_RESULT_MEDIA_MEDIAID_NOT_MATCH = 8
REC_RESULT_AFTER_FORKED = 128
#REC_RESULT_AFTER_FORKED_SUCCESS = REC_RESULT_AFTER_FORKED | REC_RESULT_SUCCESS
REC_RESULT_AFTER_FORKED_FAILED = REC_RESULT_AFTER_FORKED | REC_RESULT_FAILED

def __gen_mediaid():
    return str(uuid.uuid1())

def __get_macaddr():
    return hex(int(uuid.getnode()))[-12:]

def __umount_media():
    returncode, outs = __exec_cmd([UMOUNT_CMD, os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO)])
    #if returncode !=0:
    #    return 1
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT_MEDIA])
    #if returncode !=0:
    #    return 1
    return 0

def __umount_share():
    returncode, outs = __exec_cmd([UMOUNT_CMD, os.path.join(ROOTFS_MNT_SHARE, MEDIAINFO)])
    #if returncode !=0:
    #    return 1
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT_SHARE])
    #if returncode !=0:
    #    return 1
    return 0

def fake_blank(dev, lock_file, finished_handler_path, verify=True, check_hash=True, dev_alt_name=None, src_alt_name=None):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    if not dev_alt_name:
        dev_alt_name = DEV_ALT_NAME_DEFAULT
    if not src_alt_name:
        src_alt_name = SRC_ALT_NAME_DEFAULT
    if bdrec.get_dev_status(dev) != bdrec.DEV_STATUS_READY:
        print("device not ready", file=sys.stderr)
        return REC_RESULT_DEV_NOT_READY
    if disc_status(dev) == bdrec.DISC_STATUS_BLANK:
        print("already blank_media.", file=sys.stderr)
        return REC_RESULT_FAILED
    flocker = FileLocker(LOCK_FILE, True)
    #__umount_media()
    #__makedirs(ROOTFS_MNT_MEDIA, exist_ok=True)
    #__umount_share()
    #__makedirs(ROOTFS_MNT_SHARE, exist_ok=True)
    if os.path.exists(__to_fspath(ROOTFS)):
        __rmtree(ROOTFS)
    __makedirs(os.path.join(ROOTFS, MEDIAINFO))
    if __set_rec_type(dev, REC_TYPE_FAKE_BLANK) != 0:
        print("__set_rec_type failed", file=sys.stderr)
        return REC_RESULT_FAILED

    def fake_blank_started():
        flocker.unlock()

    def fake_blank_finished():
        __umount_share()

    result = bdrec.start(dev, lock_file, VOLID_FAKE_BLANK, ROOTFS, False, None, finished_handler_path, REC_TYPE_FAKE_BLANK, verify=verify, check_hash=check_hash, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, force=True, func_started=fake_blank_started, func_finished=fake_blank_finished)
    if result != 0:
        print("bdrec.start failed", file=sys.stderr)
        if result == bdrec.START_AFTER_FORKED_FAILED:
            return REC_RESULT_AFTER_FORKED_FAILED
        return REC_RESULT_FAILED
    return REC_RESULT_SUCCESS

def format(dev, lock_file, volid, fiscal_year, finished_handler_path, dev_alt_name=None, src_alt_name=None, force=False):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    if not dev_alt_name:
        dev_alt_name = DEV_ALT_NAME_DEFAULT
    if not src_alt_name:
        src_alt_name = SRC_ALT_NAME_DEFAULT
    if bdrec.get_dev_status(dev) != bdrec.DEV_STATUS_READY:
        print("device not ready", file=sys.stderr)
        return REC_RESULT_DEV_NOT_READY
    if disc_status(dev) != bdrec.DISC_STATUS_BLANK:
        if not force:
            print("not blank_media", file=sys.stderr)
            print("use force option if force format", file=sys.stderr)
            return REC_RESULT_MEDIA_NOT_BLANK
    flocker = FileLocker(LOCK_FILE, True)
    #__umount_media()
    #__makedirs(ROOTFS_MNT_MEDIA, exist_ok=True)
    #__umount_share()
    #__makedirs(ROOTFS_MNT_SHARE, exist_ok=True)
    if os.path.exists(__to_fspath(ROOTFS)):
        __rmtree(ROOTFS)
    __makedirs(os.path.join(ROOTFS, MEDIAINFO))
    __touch(os.path.join(ROOTFS, MEDIAINFO, FISCALYEAR))
    with open(__to_fspath(os.path.join(ROOTFS, MEDIAINFO, FISCALYEAR)), "r+") as f:
        f.write(fiscal_year)
    mediaid = __gen_mediaid()
    __touch(os.path.join(ROOTFS, MEDIAINFO, MEDIAID))
    with open(__to_fspath(os.path.join(ROOTFS, MEDIAINFO, MEDIAID)), "r+") as f:
        f.write(mediaid)
    if __set_rec_type(dev, REC_TYPE_FORMAT) != 0:
        print("__set_rec_type failed", file=sys.stderr)
        return REC_RESULT_FAILED

    def format_started():
        flocker.unlock()

    def format_finished():
        __umount_share()

    result = bdrec.start(dev, lock_file, volid, ROOTFS, False, None, finished_handler_path, REC_TYPE_FORMAT, verify=False, check_hash=False, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, force=force, func_started=format_started, func_finished=format_finished)
    if result != 0:
        print("bdrec.start failed", file=sys.stderr)
        if result == bdrec.START_AFTER_FORKED_FAILED:
            return REC_RESULT_AFTER_FORKED_FAILED
        return REC_RESULT_FAILED
    return REC_RESULT_SUCCESS

def get_format(dev):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    if bdrec.get_dev_status(dev) != bdrec.DEV_STATUS_READY:
        print("device not ready", file=sys.stderr)
        return None
    status = bdrec.get_rec_status(dev)
    if status == bdrec.REC_STATUS_RECORDING:
        print("rec_status may still recording", file=sys.stderr)
        return None
    flocker = FileLocker(LOCK_FILE, False)
    if not flocker.try_lock():
        print("failed to lock", file=sys.stderr)
        return None
    __umount_media()
    __makedirs(ROOTFS_MNT_MEDIA, exist_ok=True)
    format = {}
    format["volid"] = bdrec.get_volid(dev)
    if ENABLE_FAKE_DISC:
        if format["volid"] == VOLID_FAKE_BLANK:
            return None
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "ro,noatime,unhide", dev, ROOTFS_MNT_MEDIA])
    if returncode != 0:
        print("mount failed", file=sys.stderr)
        return format
    if os.path.exists(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, FISCALYEAR))):
        with open(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, FISCALYEAR))) as f:
            format["fiscal_year"] = f.read()
    if os.path.exists(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, MEDIAID))):
        with open(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, MEDIAID))) as f:
            format["mediaid"] = f.read()
    if os.path.exists(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, CLOSEDISC))):
        with open(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, CLOSEDISC))) as f:
            format["close_disc"] = f.read()
    if os.path.exists(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, DATETIME))):
        with open(__to_fspath(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO, DATETIME))) as f:
            format["datetime"] = f.read()
    __umount_media()
    return format

def write(dev, lock_file, fiscal_year, dir_path, close_disc, log_dir_path, finished_handler_path, verify=True, check_hash=True, dev_alt_name=None, src_alt_name=None, force=False):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    if not log_dir_path:
        log_dir_path = LOG_DIR_DEFAULT
    if not dev_alt_name:
        dev_alt_name = DEV_ALT_NAME_DEFAULT
    if not src_alt_name:
        src_alt_name = SRC_ALT_NAME_DEFAULT
    if bdrec.get_dev_status(dev) != bdrec.DEV_STATUS_READY:
        print("device not ready", file=sys.stderr)
        return REC_RESULT_DEV_NOT_READY
    if disc_status(dev) != bdrec.DISC_STATUS_APPENDABLE:
        if not force:
            print("disc status not appendable", file=sys.stderr)
            print("BD-RE may be written even if this status not appendable?",
                    file=sys.stderr)
            print("use force option if force write", file=sys.stderr)
            return REC_RESULT_MEDIA_NOT_APPENDABLE
    format = get_format(dev)
    if not format:
        print("media not have format", file=sys.stderr)
        return REC_RESULT_MEDIA_INVALID_FORMAT
    if "close_disc" in format:
        if not force:
            print("media already closed", file=sys.stderr)
            print("use force option if force write", file=sys.stderr)
            return REC_RESULT_MEDIA_ALREADY_CLOSED
    if "volid" not in format:
        print("media not have volid", file=sys.stderr)
        return REC_RESULT_MEDIA_INVALID_FORMAT
    if "fiscal_year" not in format:
        print("media not have fiscal_year", file=sys.stderr)
        return REC_RESULT_MEDIA_INVALID_FORMAT
    if format["fiscal_year"] != fiscal_year:
        print("media fiscal year not match", file=sys.stderr)
        return REC_RESULT_MEDIA_FISCAL_YEAR_NOT_MATCH
    if "mediaid" not in format:
        print("media not have mediaid", file=sys.stderr)
        return REC_RESULT_MEDIA_INVALID_FORMAT
    if format["mediaid"][-12:] != __get_macaddr():
        print("media has not format with this system", file=sys.stderr)
        return REC_RESULT_MEDIA_MEDIAID_NOT_MATCH
    flocker = FileLocker(LOCK_FILE, True)
    __umount_media()
    __makedirs(ROOTFS_MNT_MEDIA, exist_ok=True)
    __umount_share()
    __makedirs(ROOTFS_MNT_SHARE, exist_ok=True)
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "ro,noatime,unhide", dev, ROOTFS_MNT_MEDIA])
    if returncode !=0:
        print("mount failed", file=sys.stderr)
        return REC_RESULT_FAILED
    if os.path.exists(__to_fspath(ROOTFS)):
        __rmtree(ROOTFS)
    __makedirs(ROOTFS)
    __copytree(os.path.join(ROOTFS_MNT_MEDIA, MEDIAINFO), os.path.join(ROOTFS, MEDIAINFO))
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT_MEDIA])
    if returncode !=0:
        print("umount failed", file=sys.stderr)
        return REC_RESULT_FAILED
    if close_disc:
        __touch(os.path.join(ROOTFS, MEDIAINFO, CLOSEDISC))
        with open(__to_fspath(os.path.join(ROOTFS, MEDIAINFO, CLOSEDISC)), "r+") as f:
            f.write("closed")
    __touch(os.path.join(ROOTFS, MEDIAINFO, DATETIME))
    with open(__to_fspath(os.path.join(ROOTFS, MEDIAINFO, DATETIME)), "r+") as f:
        f.write(str(datetime.datetime.now()))
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "bind", dir_path, ROOTFS_MNT_SHARE])
    if returncode !=0:
        print("bind mount failed", file=sys.stderr)
        return REC_RESULT_FAILED
    if not os.path.exists(__to_fspath(os.path.join(ROOTFS_MNT_SHARE, MEDIAINFO))):
        __makedirs(os.path.join(ROOTFS_MNT_SHARE, MEDIAINFO))
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "bind",
                                   os.path.join(ROOTFS,           MEDIAINFO),
                                   os.path.join(ROOTFS_MNT_SHARE, MEDIAINFO)])
    if returncode !=0:
        print("bind mount failed", file=sys.stderr)
        return REC_RESULT_FAILED
    if close_disc:
        rec_type = REC_TYPE_WRITE_WITH_CLOSE
    else:
        rec_type = REC_TYPE_WRITE
    if __set_rec_type(dev, rec_type) != 0:
        print("__set_rec_type failed", file=sys.stderr)
        return REC_RESULT_FAILED

    def write_started():
        flocker.unlock()

    def write_finished():
        __umount_share()

    result = bdrec.start(dev, lock_file, format["volid"], ROOTFS_MNT_SHARE, close_disc, log_dir_path, finished_handler_path, rec_type, verify=verify, check_hash=check_hash, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, force=force, func_started=write_started, func_finished=write_finished)
    if result != 0:
        print("bdrec.start failed", file=sys.stderr)
        if result == bdrec.START_AFTER_FORKED_FAILED:
            return REC_RESULT_AFTER_FORKED_FAILED
        return REC_RESULT_FAILED
    return REC_RESULT_SUCCESS

def __get_rec_type(dev):
    rec_type = None
    flocker = FileLocker(FILE_REC_TYPE, True)
    rec_types = flocker.read()
    if rec_types:
        status_list = rec_types.split(',')
        if status_list[0] != dev:
            print("%s not match %s" % (status_list[0], dev), file=sys.stderr)
            return None
        rec_type = status_list[1]
    else:
        return None
    return rec_type

def __set_rec_type(dev, type):
    flocker = FileLocker(FILE_REC_TYPE, True)
    if not type:
        return 1
    try:
        flocker.write(','.join((dev, type)))
    except:
        return 1
    return 0

def rec_status(dev):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    status = bdrec.get_rec_status(dev)
    rec_type = __get_rec_type(dev)
    if rec_type:
        status = "%s, %s" % (status, rec_type)
    return status

def dev_status(dev):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    status = bdrec.get_dev_status(dev)
    return status

def disc_status(dev):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    status = bdrec.get_disc_status(dev)
    if ENABLE_FAKE_DISC:
        if status != bdrec.DISC_STATUS_BLANK:
            if bdrec.get_volid(dev) == VOLID_FAKE_BLANK:
                return bdrec.DISC_STATUS_BLANK
        format = get_format(dev)
        if format:
            if "close_disc" not in format:
                return bdrec.DISC_STATUS_APPENDABLE
            else:
                return bdrec.DISC_STATUS_COMPLETE
    return status

def eject(dev, close_tray=False, force=False):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    status = bdrec.get_rec_status(dev)
    if status == bdrec.REC_STATUS_RECORDING:
        if not force:
            print("rec_status may still recording", file=sys.stderr)
            print("use force option if force eject", file=sys.stderr)
            return 1
    flocker = FileLocker(LOCK_FILE, True)
    __umount_media()
    result = bdrec.eject(dev, close_tray=close_tray, force=force)
    if result != 0:
        print("bdrec.eject failed", file=sys.stderr)
        return result
    return 0

def get_last_written_summary(dev, volid, log_dir_path):
    if not dev:
        dev = bdrec.DEV_DEFAULT
    if not log_dir_path:
        log_dir_path = LOG_DIR_DEFAULT
    if not volid:
        format = get_format(dev)
        if not format:
            return None
        if "volid" not in format:
            return None
        volid = format["volid"]
    return bdrec.get_last_written_summary(volid, log_dir_path)

if __name__ == '__main__':
    __init()
    usage = "usage: %prog [options] fake_blank|format|write|rec_status|dev_status|disc_status|get_format|eject|close_tray|get_last_written_summary|check_writable_volid"
    parser = OptionParser(usage)
    parser.add_option("-d", "--dev", dest="dev", help="dev", default=None, action="store", type="string")
    parser.add_option("-l", "--lock_file", dest="lock_file", help="lock_file", default=None, action="store", type="string")
    parser.add_option("-i", "--volid", dest="volid", help="volid", default=None, action="store", type="string")
    parser.add_option("-y", "--fiscal_year", dest="fiscal_year", help="fiscal_year", default=None, action="store", type="string")
    parser.add_option("-p", "--dir_path", dest="dir_path", help="dir_path", default=None, action="store", type="string")
    parser.add_option("-c", "--close_disc", dest="close_disc", help="close_disc", default=False, action="store_true")
    parser.add_option("-L", "--log_dir_path", dest="log_dir_path", help="log_dir_path", default=None, action="store", type="string")
    parser.add_option("-D", "--dev_alt_name", dest="dev_alt_name", help="dev_alt_name", default=None, action="store", type="string")
    parser.add_option("-S", "--src_alt_name", dest="src_alt_name", help="src_alt_name", default=None, action="store", type="string")
    parser.add_option("-H", "--finished_handler_path", dest="finished_handler_path", help="finished_handler_path", default=None, action="store", type="string")
    parser.add_option("-v", "--verify", dest="verify", help="verify", default=True, action="store_true")
    parser.add_option("-C", "--check_hash", dest="check_hash", help="check_hash", default=True, action="store_true")
    parser.add_option("-f", "--force", dest="force", help="force", default=False, action="store_true")
    (options, args) = parser.parse_args()

    if not args or len(args) != 1:
        parser.print_help()
        exit(1)

    command = args[0]

    if command == "fake_blank":
        result = fake_blank(options.dev, options.lock_file, options.finished_handler_path, verify=options.verify, dev_alt_name=options.dev_alt_name, src_alt_name=options.src_alt_name)
        exit(result)
    elif command == "format":
        if not options.volid:
            print("Require --volid.")
            exit(1)
        if not options.fiscal_year:
            print("Require --fiscal_year.")
            exit(1)
        result = format(options.dev, options.lock_file, options.volid, options.fiscal_year, options.finished_handler_path, dev_alt_name=options.dev_alt_name, src_alt_name=options.src_alt_name, force=options.force)
        exit(result)
    elif command == "write":
        if not options.fiscal_year:
            print("Require --fiscal_year.")
            exit(1)
        if not options.dir_path:
            print("Require --dir_path.")
            exit(1)
        result = write(options.dev, options.lock_file, options.fiscal_year, options.dir_path, options.close_disc, options.log_dir_path, options.finished_handler_path, verify=options.verify, dev_alt_name=options.dev_alt_name, src_alt_name=options.src_alt_name, force=options.force)
        exit(result)
    elif command == "rec_status":
        status = rec_status(options.dev)
        if not status:
            exit(1)
        print(status)
        exit(0)
    elif command == "dev_status":
        result = dev_status(options.dev)
        if not result:
            exit(1)
        print(result)
        exit(0)
    elif command == "disc_status":
        status = disc_status(options.dev)
        if not status:
            exit(1)
        print(status)
        exit(0)
    elif command == "get_format":
        format = get_format(options.dev)
        if not format:
            exit(1)
        print(__to_keyvalue_str(format), end="")
        exit(0)
    elif command == "eject":
        if eject(options.dev, force=options.force) != 0:
            exit(1)
        exit(0)
    elif command == "close_tray":
        if eject(options.dev, close_tray=True, force=options.force) != 0:
            exit(1)
        exit(0)
    elif command == "get_last_written_summary":
        last_written_summary = get_last_written_summary(dev=options.dev, volid=options.volid, log_dir_path=options.log_dir_path)
        if not last_written_summary:
            exit(1)
        print(__to_keyvalue_str(last_written_summary), end="")
        exit(0)
    elif command == "check_writable_volid":
        if not options.volid:
            print("Require --volid.")
            exit(1)
        if bdrec.check_writable_volid(options.volid) != 0:
            exit(1)
        exit(0)

    exit(1)
