#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from ionas.request import Request, Entry, EntryList, XmlConfNode
from product_conf import SUDO_CMD
from subprocess import Popen, PIPE
from pkgaddon.meimeikun_config import MEIMEIKUN_CONFIG_COMMAND
from .disk.gettext import gettext

import json


def send_entry(falcmd, entry, raise_exception=False):
    try:
        entrylist = EntryList()
        entrylist.append_entry(entry)
        req = Request(falcmd, filter_cmd=SUDO_CMD)
        res = req.request(entrylist)
        return res.get_entrylist().get_entry()
    except Exception as err:
        if raise_exception:
            raise
        return None


def _execute_cmd(operation, data=None, sudo=False):
    cmd = [MEIMEIKUN_CONFIG_COMMAND, operation]
    if data:
        cmd.append(data)
    if sudo:
        cmd = [SUDO_CMD] + cmd

    proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()

    if proc.returncode != 0:
        raise Exception(stderr)  # TODO

    return stdout


def load_config():
    try:
        config_str = _execute_cmd("load", sudo=True)
        return json.loads(config_str)
    except Exception as err:
        return {}


def update_config(config_dict):
    try:
        _execute_cmd("update", json.dumps(config_dict), sudo=True)
    except Exception as err:
        msg = gettext("ERR_MEIMEIKUN_CHANGE")
        return False, msg

    msg = gettext("MSG_MEIMEIKUN_CHANGE")
    return True, msg


def get_path_of_protected_fiscal_year_folder(fiscal_year):
    try:
        if isinstance(fiscal_year, int):
            fiscal_year = str(fiscal_year)

        path = _execute_cmd("get_path_of_protected_fiscal_year_folder",
                            fiscal_year, sudo=True)
        return path.replace("\n", "")
    except Exception as err:
        return None
